%%% Local Projections

close all; clear; clc;

%% Settings and data upload:

set(0,'defaultAxesFontName', 'Times');
set(0,'defaultAxesLineStyleOrder','-|--|:', 'defaultLineLineWidth',1.5)

rng('default');
rng(0);     

addpath('Functions')
addpath('Data')

Makedata;

%% First step - Estimate SVAR to estimate the financial shock:

vardata=[ip(2:end),diff(cpi),unrate(2:end),ebp(2:end),NFCI(2:end),sp500(2:end),ffr(2:end)]; % Baseline variables
vardata=vardata(1:end-24,:);

pos_shock=4; % position of the financial shock

n=size(vardata,2); % # of variables
p=5; % # lags 
c=1; % including constant
hor=49; % # horizons of IRFs + 1
cum=0; % IRFs not cumulated

% Financial shocks:

[~,~,~,~,~,~,~,~,~,~,eta]=chol_irf(vardata,n,p,c,hor); % eta = Cholesky shocks

finshock=eta(:,pos_shock); % financial shock
finshock_squared=eta(:,pos_shock).^2; % financial shock squared

%% Local projection:

finaldata=vardata(p+1:end,:); % endogenous variables of VARX, same as first step

news=[finshock finshock_squared]; % exogenous variables of VARX

hor=49; % horizon of the IRFs
c=1; % include a constant
lags=1; % # of lags of the dependent variable

controls=[]; % set of controls
numcontrols=size(controls,2);

y=finaldata; % data in levels

% Set up vectors for the IRFs and the standard errors to compute the bands:

bands_beta=zeros(hor,size(y,2)); % standard errors for the bands
beta=zeros(hor,size(y,2)); % OLS estimate of beta_{0}^{h}

bands_betanl=zeros(hor,size(y,2)); % standard errors for the bands
betanl=zeros(hor,size(y,2)); % OLS estimate of beta_{0}^{h}

for i=1:size(y,2) % IRFs for FEDFUNDS, GDPC1, GDPDEF and UNRATE
    
[lp,bands]=LP(y(:,i),lags,controls,news,hor,c);

bands_beta(:,i)=bands(c+lags+numcontrols+1,:)';
beta(:,i)=lp(c+lags+numcontrols+1,:)';

bands_betanl(:,i)=bands(c+lags+numcontrols+2,:)';
betanl(:,i)=lp(c+lags+numcontrols+2,:)';

end

figure;
colorBNDS1=[0.75 0.75 0.75];
colorBNDS2=[0.9 0.9 0.9];
names={'INDPRO'; 'CPI Inflation'; 'UNRATE'; 'EBP'; 'NFCI'; 'SP500'; 'FFR';'GS10'};
HighH1=beta+1*bands_beta;
LowH1=beta-1*bands_beta;
HighH2=beta+1.65*bands_beta;
LowH2=beta-1.65*bands_beta;

for j=1:size(beta,2)
    subplot(size(beta,2),1,j)
    fill([0:length(beta(1:end,:))-1 fliplr(0:length(beta(1:end,:))-1)]' ,[(HighH2(1:end,j)) ; flipud((LowH2(1:end,j)))],...
        colorBNDS2,'EdgeColor','None'); hold on;
    fill([0:length((beta(1:end,:)))-1 fliplr(0:length(beta(1:end,:))-1)]' ,[(HighH1(1:end,j)) ; flipud((LowH1(1:end,j)))],...
        colorBNDS1,'EdgeColor','None'); hold on;
    plot(0:length(beta(1:end,:))-1,(beta(1:end,j)),'LineWidth',3.5,'Color','k'); hold on;
    if j==1
        title('Financial Shock','FontWeight','bold');
    end
     ylabel(names(j))
    line(get(gca,'Xlim'),[0 0],'Color','k','LineStyle','--','LineWidth',1.5); hold off;
    xlim([0 length(beta(1:end,:))-1]);
    set(gca,'FontSize',18)
end

figure;
colorBNDS1=[0.75 0.75 0.75];
colorBNDS2=[0.9 0.9 0.9];
names={'INDPRO'; 'CPI Inflation'; 'UNRATE'; 'EBP'; 'NFCI'; 'SP500'; 'FFR';'GS10'};
HighH1=betanl+1*bands_betanl;
LowH1=betanl-1*bands_betanl;
HighH2=betanl+1.65*bands_betanl;
LowH2=betanl-1.65*bands_betanl;

for j=1:size(beta,2)
    subplot(size(beta,2),1,j)
    fill([0:length(beta(1:end,:))-1 fliplr(0:length(betanl(1:end,:))-1)]' ,[(HighH2(1:end,j)) ; flipud((LowH2(1:end,j)))],...
        colorBNDS2,'EdgeColor','None'); hold on;
    fill([0:length((beta(1:end,:)))-1 fliplr(0:length(betanl(1:end,:))-1)]' ,[(HighH1(1:end,j)) ; flipud((LowH1(1:end,j)))],...
        colorBNDS1,'EdgeColor','None'); hold on;
    plot(0:length(beta(1:end,:))-1,(betanl(1:end,j)),'LineWidth',3.5,'Color','k'); hold on;
    if j==1
        title('Financial Shock Squared','FontWeight','bold');
    end
     ylabel(names(j))
    line(get(gca,'Xlim'),[0 0],'Color','k','LineStyle','--','LineWidth',1.5); hold off;
    xlim([0 length(betanl(1:end,:))-1]);
    set(gca,'FontSize',18)
end
