%%% Robustness with absolute value - Nonlinear Financial Shocks
%%% Authors: Mario Forni, Luca Gambetti, Nicol Maffei Faccioli, and Luca Sala

close all; clear; clc;

%% Settings and data upload:

set(0,'defaultAxesFontName', 'Times');
set(0,'defaultAxesLineStyleOrder','-|--|:', 'defaultLineLineWidth',1.5)

rng('default');
rng(0);     

addpath('Functions')
addpath('Data')

Makedata;

%% First step - Estimate SVAR to estimate the financial shock:

vardata=[ip(2:end),diff(cpi),unrate(2:end),ebp(2:end),NFCI(2:end),sp500(2:end),ffr(2:end)]; % Baseline variables
vardata=vardata(1:end-24,:);

pos_shock=4; % position of the financial shock

n=size(vardata,2); % # of variables
p=5; % # lags 
c=1; % including constant
hor=49; % # horizons of IRFs + 1
cum=0; % IRFs not cumulated

% Financial shocks:

[~,~,~,~,~,~,~,~,~,~,eta]=chol_irf(vardata,n,p,c,hor); % eta = Cholesky shocks

finshock=eta(:,pos_shock); % financial shock
finshock_abs=abs(eta(:,pos_shock)); % financial shock absolute value

%% Second step - Estimate a SVARX:

finaldata=vardata(p+1:end,:); % endogenous variables of VARX, same as first step

news=[finshock finshock_abs]; % exogenous variables of VARX

nonlin=0; % Type of nonlinearity (square=1, abs=0)

restr=3; % restrict no impact effect for variables before EBP for the shock and its square, and no impact effect on EBP for the squared shock
%restr=2; % restrict only no impact effect for variables before EBP for the linear shock, no restriction on the square

m=size(news,2); % # of exogenous variables
q=0;  % # of lags of the exogenous variables

SVARX; % estimation of the SVARX

%% IRFs plots:

% IRFs plot - Figure 3:

colorBNDS=[0.9 0.9 0.9];
colorBNDS68=[0.75 0.75 0.75]; 

VARnames={'INDPRO'; 'CPI Inflation'; 'UNRATE'; 'EBP'; 'NFCI'; 'SP500'; 'FFR';'Uncertainty'};
Shocknames={'Financial Shock'; 'Financial Shock Squared'};

irf_plot_VARX(C_wold,HighC90,LowC90,HighC,LowC,VARnames,Shocknames,colorBNDS,colorBNDS68,hor,n,m,pos_shock)

%% Plots by negative vs positive:

plots_posneg_abs;

%% Merge the two plots - Figure 7:

figure;

m=3;
Shocknames={'Financial Shock'; 'Financial Shock Absolute Value'; 'Overall Financial Shock'};

MiddleD=NaN(hor,n*m);
MiddleD_neg=NaN(hor,n*m);
HighD=NaN(hor,n*m);
HighD68=NaN(hor,n*m);
LowD=NaN(hor,n*m);
LowD68=NaN(hor,n*m);

for i=[1:m:n*m;1:m-1:n*(m-1);1:n]

MiddleD(:,i(1):i(1)+m-1)=[C_wold(:,i(2):i(2)+m-2) C_pos(:,i(3))];
MiddleD_neg(:,i(1):i(1)+m-1)=[C_wold(:,i(2):i(2)+m-2) C_neg(:,i(3))];

HighD(:,i(1):i(1)+m-1)=[HighC90(:,i(2):i(2)+m-2) HighC90_pos(:,i(3))];
LowD(:,i(1):i(1)+m-1)=[LowC90(:,i(2):i(2)+m-2) LowC90_pos(:,i(3))];
HighD68(:,i(1):i(1)+m-1)=[HighC(:,i(2):i(2)+m-2) HighC_pos(:,i(3))];
LowD68(:,i(1):i(1)+m-1)=[LowC(:,i(2):i(2)+m-2) LowC_pos(:,i(3))];

end

for i=1:m
    
for k=[1:m:n*m;1:n]
    
    subplot(n,m,k(1)+(i-1))
    s1=fill([0:hor-1 fliplr(0:hor-1)]' ,[HighD(:,k(1)+(i-1)); flipud(LowD(:,k(1)+(i-1)))],...
        colorBNDS,'EdgeColor','None'); hold on; 
    s2=fill([0:hor-1 fliplr(0:hor-1)]' ,[HighD68(:,k(1)+(i-1)); flipud(LowD68(:,k(1)+(i-1)))],...
        colorBNDS68,'EdgeColor','None'); hold on; %grid on; %grid minor;
    p1=plot(0:hor-1,MiddleD(:,k(1)+(i-1)),'LineWidth',3.5,'Color','k'); hold on;
    if i==3
    p2=plot(0:hor-1,MiddleD_neg(:,k(1)+(i-1)),'LineWidth',3.5,'Color','r','LineStyle',':'); hold on;    
    end
    set(gca,'FontSize',18)
    line(get(gca,'Xlim'),[0 0],'Color','k','LineStyle','--','LineWidth',1.5); hold off;
    ylabel(strcat(VARnames{k(2)}), 'FontSize', 18);
    if k(1) == 1
        title(strcat(Shocknames{i}), 'FontSize', 18);
    end
    axis tight
    xlim([0 hor-1]);
    
end

end

legend([s1,s2,p1,p2],{'90% confidence bands','68% confidence bands','IRF positive','IRF negative'},'FontSize',18,'Orientation','Horizontal')
    
