%%% Main code - Nonlinear Financial Shocks
%%% Authors: Mario Forni, Luca Gambetti, Nicolo' Maffei Faccioli, and Luca Sala

close all; clear; clc;

%% Settings and data upload:

set(0,'defaultAxesFontName', 'Times');
set(0,'defaultAxesLineStyleOrder','-|--|:', 'defaultLineLineWidth',1.5)

rng('default');
rng(0);     

addpath('Functions')
addpath('Data')

Makedata;

%% First step - Estimate SVAR to estimate the financial shock:

vardata=[ip(2:end),diff(cpi),unrate(2:end),ebp(2:end),sp500(2:end),ffr(2:end)]; % Baseline variables
vardata=vardata(1:end-24,:);

pos_shock=4; % position of the financial shock

n=size(vardata,2); % # of variables
p=5; % # lags 
c=1; % including constant
hor=49; % # horizons of IRFs + 1
cum=0; % IRFs not cumulated

% Financial shocks:

[D,~,~,~,~,~,~,~,~,~,eta]=chol_irf(vardata,n,p,c,hor); % eta = Cholesky shocks

finshock=eta(:,pos_shock); % financial shock
finshock_squared=eta(:,pos_shock).^2; % financial shock squared

% Shocks plot - Figure 2:

figure;
%dates=1973+7/12:1/12:2021+12/12;
dates=1973+7/12:1/12:2019+12/12;
subplot(2,1,1)
plot(dates,finshock,'-','LineWidth',2,'Color',[0.5 0.5 0.5]); axis tight;
title('Financial shock, $u_{ft}$','Interpreter','Latex')
set(gca,'FontSize',18)
subplot(2,1,2)
plot(dates,finshock_squared,'-','LineWidth',2,'Color',[0.2 0.2 0.2]); axis tight;
title('Financial shock squared, $g(u_{ft})=u_{ft}^2$','Interpreter','Latex')
set(gca,'FontSize',18)

%% Second step - Estimate a SVARX:

finaldata=vardata(p+1:end,:); % endogenous variables of VARX, same as first step

news=[finshock finshock_squared]; % exogenous variables of VARX

nonlin=1; % Type of nonlinearity (square=1, abs=0)

restr=3; % restrict no impact effect for variables before EBP for the shock and its square, and no impact effect on EBP for the squared shock
%restr=2; % restrict only no impact effect for variables before EBP for the linear shock, no restriction on the square

m=size(news,2); % # of exogenous variables
q=0;  % # of lags of the exogenous variables

SVARX; % estimation of the SVARX

%% IRFs plots:

% IRFs plot - Figure 3:

colorBNDS=[0.9 0.9 0.9];
colorBNDS68=[0.75 0.75 0.75]; 

VARnames={'INDPRO'; 'CPI Inflation'; 'UNRATE'; 'EBP'; 'SP500'; 'FFR';'GS10'};
Shocknames={'Financial Shock'; 'Financial Shock Squared'};

irf_plot_VARX(C_wold,HighC90,LowC90,HighC,LowC,VARnames,Shocknames,colorBNDS,colorBNDS68,hor,n,m,pos_shock)

% IRFs plot - negative vs positive shocks by magnitude - Figure 4:

plots_posneg_square;
