%%% Main code - Nonlinear Financial Shocks
%%% Authors: Mario Forni, Luca Gambetti, Nicol Maffei Faccioli, and Luca Sala

close all; clear; clc;

%% Settings and data upload:

set(0,'defaultAxesFontName', 'Times');
set(0,'defaultAxesLineStyleOrder','-|--|:', 'defaultLineLineWidth',1.5)

rng('default');
rng(0);     

addpath('Functions')
addpath('Data')

Makedata;

%% First step - Estimate SVAR to estimate the financial shock:

vardata=[ip(2:end),diff(cpi),unrate(2:end),ebp(2:end),NFCI(2:end),sp500(2:end),ffr(2:end)]; % Baseline variables
vardata=vardata(1:end-24,:);

pos_shock=4; % position of the financial shock

n=size(vardata,2); % # of variables
p=5; % # lags 
c=1; % including constant
hor=49; % # horizons of IRFs + 1
cum=0; % IRFs not cumulated

% Financial shocks:

[~,~,~,~,~,~,~,~,~,~,eta]=chol_irf(vardata,n,p,c,hor); % eta = Cholesky shocks

finshock=eta(:,pos_shock); % financial shock
finshock_squared=eta(:,pos_shock).^2; % financial shock squared

%% Second step - Estimate a SVARX:

finaldata=vardata(p+1:end,:);

n=size(vardata,2); % # of variables

news=[finshock finshock_squared]; % BASELINE square

restr=3; % restrict no impact effect for variables before EBP for the shock and its square, and no impact effect on EBP for the squared shock
%restr=2; % restrict only no impact effect for variables before EBP for the linear shock, no restriction on the square

m=size(news,2); % # of exogenous variables
q=5;  % # of lags of the exogenous variables

squareonly=1;

SVARX_lagssquare;

colorBNDS=[0.9 0.9 0.9]; % Set color of the bands
colorBNDS68=[0.75 0.75 0.75]; % Set color of the bands

VARnames={'INDPRO'; 'CPI Inflation'; 'UNRATE'; 'EBP'; 'NFCI'; 'SP500'; 'FFR';'Uncertainty'};
Shocknames={'Financial Shock'; 'Financial Shock Squared'};

%% IRFs plot:

irf_plot_VARX(C_wold,HighC90,LowC90,HighC,LowC,VARnames,Shocknames,colorBNDS,colorBNDS68,hor,n,m,pos_shock)

%% IRFs plot - negative vs positive shocks by magnitude:

plots_posneg_square_lagssquare;
