%%% Main code with state-dependence - Nonlinear Financial Shocks
%%% Authors: Luca Gambetti, Nicolo' Maffei Faccioli, and Luca Sala

close all; clear; clc;       

%% Settings and data upload:

rng('default');
rng(0);     

addpath('Functions')
addpath('Data')

Makedata;

%% First step - Estimate SVAR to estimate the financial shock:

vardata=[ip(2:end),diff(cpi),unrate(2:end),ebp(2:end),NFCI(2:end),sp500(2:end),ffr(2:end)]; % Baseline variables
vardata=vardata(1:end-24,:);

pos_shock=4; % position of the financial shock

n=size(vardata,2); % # of variables
p=5; % # lags 
c=1; % including constant
hor=49; % # horizons of IRFs + 1
cum=0; % IRFs not cumulated

% Financial shocks:

[~,~,~,~,~,~,~,~,~,~,eta]=chol_irf(vardata,n,p,c,hor); % eta = Cholesky shocks

finshock=eta(:,pos_shock); % financial shock
finshock_squared=eta(:,pos_shock).^2; % financial shock squared

%% Estimate a SVARX:

finaldata=vardata(p+1:end,:);

% State-dependence:

d=zeros(length(finshock),1);
d(macrouncertainty(p+1:end-1-24)>mean(macrouncertainty(p+1:end-1-24)))=1; % Macro Uncertainty

% d=zeros(length(finshock),1);
% d(NFCI(p+1:end-1-24)>mean(NFCI(p+1:end-1-24)))=1; % NFCI

% d=zeros(length(finshock),1);
% d(GS10state(p+1:end-1-24)>0)=1; % Tightening cycles

% d=zeros(length(finshock),1);
% d(unrate(p+1:end-1-24)>mean(unrate(p+1:end-1-24)))=1; % Boom vs Bust

news=[finshock finshock_squared d.*finshock]; 

restr=3;
nonlin=1;
endo=5; % order of the endogenous variable if used in the indicator, 0 if not included in VAR

m=size(news,2); % # of exogenous variables
q=0;  % # of lags of the exogenous variables

SVARX_state;

colorBNDS=[0.9 0.9 0.9]; % Set color of the bands
colorBNDS68=[0.75 0.75 0.75]; % Set color of the bands

VARnames={'INDPRO'; 'CPI'; 'UNRATE'; 'EBP'; 'NFCI'; 'SP500'; 'FFR';'Uncertainty'};
Shocknames={'Financial'; 'Financial squared';'High uncertainty'};

irf_plot_VARX(C_wold,HighC90,LowC90,HighC,LowC,VARnames,Shocknames,colorBNDS,colorBNDS68,hor,n,m,pos_shock)
