%%% Main code - Nonlinear Financial Shocks
%%% Authors: Mario Forni, Luca Gambetti, Nicolo' Maffei Faccioli, and Luca Sala

close all; clear; clc;

%% Settings and data upload:

set(0,'defaultAxesFontName', 'Times');
set(0,'defaultAxesLineStyleOrder','-|--|:', 'defaultLineLineWidth',1.5)

rng('default');
rng(0);     

addpath('Functions')
addpath('Data')

Makedata;

%% First step - Estimate SVAR to estimate the financial shock:

vardata=[ip(2:end),diff(cpi),unrate(2:end),ebp(2:end),NFCI(2:end),sp500(2:end),ffr(2:end)]; % Baseline variables
vardata=vardata(1:end-24,:);

pos_shock=4; % position of the financial shock

n=size(vardata,2); % # of variables
p=5; % # lags 
c=1; % including constant
hor=49; % # horizons of IRFs + 1
cum=0; % IRFs not cumulated

% Financial shocks:

[~,~,~,~,~,~,~,~,~,~,eta]=chol_irf(vardata,n,p,c,hor); % eta = Cholesky shocks

finshock=eta(:,pos_shock); % financial shock
finshock_squared=eta(:,pos_shock).^2; % financial shock squared

%% Second step - Estimate a SVARX:

finaldata=vardata(p+1:end,:); % endogenous variables of VARX, same as first step

news=[finshock_squared]; % exogenous variables of VARX
%news=[abs(finshock)]; % exogenous variables of VARX

nonlin=4; % Type of nonlinearity 
%nonlin=5; % Type of nonlinearity 

restr=2; 

m=size(news,2); % # of exogenous variables
q=0;  % # of lags of the exogenous variables

SVARX; % estimation of the SVARX

omega_r=cov(Y-X*pi_hat);

%% Second step - Estimate a SVARX:

finaldata=vardata(p+1:end,:); % endogenous variables of VARX, same as first step

news=[abs(finshock) finshock_squared]; % exogenous variables of VARX

nonlin=3; % Type of nonlinearity (square=1, abs=0)

restr=3; 

m=size(news,2); % # of exogenous variables
q=0;  % # of lags of the exogenous variables

SVARX; % estimation of the SVARX

omega=cov(Y-X*pi_hat);

%% Statistic:

lrt=(size(finaldata,1)-p).*(log(det(omega_r))-log(det(omega)));

lrt>chi2inv(0.95,7)

lrt

1-chi2cdf(lrt,7)